package com.example.afrishop_v3.controllers;

import com.example.afrishop_v3.base.Result;
import com.example.afrishop_v3.enums.ResultCodeEnum;
import com.example.afrishop_v3.models.TbCfUserInfo;
import com.example.afrishop_v3.payload.request.LoginRequest;
import com.example.afrishop_v3.payload.response.JwtResponse;
import com.example.afrishop_v3.payload.response.MessageResponse;
import com.example.afrishop_v3.repository.UserRepository;
import com.example.afrishop_v3.security.jwt.JwtUtils;
import com.example.afrishop_v3.security.services.UserDetailsImpl;
import com.example.afrishop_v3.util.IdUtil;
import org.springframework.http.ResponseEntity;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.web.bind.annotation.*;

import java.util.Collection;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

@CrossOrigin(origins = "*", maxAge = 3600)
@RestController
@RequestMapping("/api/auth")
public class AuthController {
	private final AuthenticationManager authenticationManager;

	private final UserRepository userRepository;


	private final PasswordEncoder encoder;

	private final JwtUtils jwtUtils;

	public AuthController(AuthenticationManager authenticationManager, UserRepository userRepository, PasswordEncoder encoder, JwtUtils jwtUtils) {
		this.authenticationManager = authenticationManager;
		this.userRepository = userRepository;
		this.encoder = encoder;
		this.jwtUtils = jwtUtils;
	}

	@PostMapping("/signin")
	public ResponseEntity<Result> authenticateUser(@RequestBody LoginRequest loginRequest) {
		Optional<TbCfUserInfo> byAccount = userRepository.findByAccount(loginRequest.getAccount());
		if( !byAccount.isPresent() ){
			return ResponseEntity.ok(new Result<>(ResultCodeEnum.VALIDATE_ERROR,"User not found"));
		}


		Authentication authentication = authenticationManager.authenticate(
				new UsernamePasswordAuthenticationToken(loginRequest.getAccount(), loginRequest.getPassword()));

		SecurityContextHolder.getContext().setAuthentication(authentication);
		String jwt = jwtUtils.generateJwtToken(authentication);
		
		UserDetailsImpl userDetails = (UserDetailsImpl) authentication.getPrincipal();
		List<String> roles = userDetails.getAuthorities().stream()
				.map(GrantedAuthority::getAuthority)
				.collect(Collectors.toList());

		return ResponseEntity.ok(new Result<>(new JwtResponse(jwt,
				userDetails.getId(),
				userDetails.getUsername(),
				userDetails.getEmail(),
				roles)));
	}

	@PostMapping("/signup")
	public ResponseEntity<?> registerUser( @RequestBody TbCfUserInfo signUpRequest) {
		Optional<TbCfUserInfo> byAccount = userRepository.findByAccount(signUpRequest.getAccount());
		if ( byAccount.isPresent() ) {
			return ResponseEntity
					.badRequest()
					.body(new MessageResponse("Error: Username is already taken!"));
		}

		Optional<TbCfUserInfo> byEmail = userRepository.findFirstByEmail(signUpRequest.getEmail());

		if ( byEmail.isPresent() ) {
			return ResponseEntity
					.badRequest()
					.body(new MessageResponse("Error: Email is already in use!"));
		}


		signUpRequest.setUserId(IdUtil.createIdbyUUID());

		signUpRequest.setPassword(encoder.encode(signUpRequest.getPassword()));


		TbCfUserInfo userInfo = userRepository.save(signUpRequest);


		return ResponseEntity.ok(new Result<>(userInfo,"User createdSuccessfully"));
	}
}
